//*****************************************************************************
//
// File:     main.cc
// Purpose:  Main routine for mixing programme.
// Author:   Michael Edwards
// Date:     7th November 1995
// Last revision: None
//
//*****************************************************************************

#include <iostream.h>
#include <string.h>
#include "mix.h"

#define VERSION "0.9"


//*****************************************************************************

// Globals.  All except <Semaphore> are set by command line flags.

// Whether to print underflow errors or not.  Set in main(). 
BOOL Silent = NO;

// Whether to use the fast mixing algorithm or not.  Set in main(). 
BOOL Fast = NO;

// How many times we had to limit the signal.  Set in soundlist::mix(). 
long Limits = 0;

// How many underflow errors we got on the audio port. Set in
// underflow_error(). 
long Underflows = 0;

// Whether to print sound information or just plain mapping data.  Set
// in main().   
BOOL Info = YES;

BOOL TestMidi = NO;

// Whether it's OK to print underflow and limit values. This is true unless
// we want to get user input.   Set in main() and mix::get_input().
BOOL OK_to_print = YES;

// The exponent that the MIDI fader value will be raised to.  Set in
// main().
double Expt = 1.0;


#ifdef SGI
// The semaphore used for access control in the forked processes. 
usema_t *Semaphore;
#endif	// SGI 

#ifdef NEXT
// The semaphore used for access control in the forked processes. 
mutex_t Semaphore;
#endif // NEXT


//*****************************************************************************

int main(int argc, char *argv[])
{
    char *mappingsfile = NULL;
    char *timestampfile = NULL;
    char *arg;			// Each argument given on the command line. 

    // Cache programme name for use with -v flag. 
    const char *prog = argv[0];

    while (--argc > 0) {
	arg = *++argv;		// Step over the programme name. 
	if (*arg == '-')	// If it's a flag, process it. 
	    switch (*++arg) {
	    case 's':
		Silent = YES;
		break;
	    case 't':
		timestampfile = *++argv;	// The next argument. 
		--argc;
		break;
	    case 'v':
		cerr << '\n' << prog << ": Version " << VERSION
		     << "\nAuthor: Michael Edwards (michael@ccrma.stanford"
		     << ".edu)\nCompiled at " << __TIME__ << " on " << __DATE__
		     << ".\n\n";
		exit(0);
	    case 'f':
		Fast = YES;
		break;
	    case 'i':
		Info = NO;
		break;
	    case 'e':
		Expt = atof(*++argv);
		--argc;
		break;
	    case 'm':
	        TestMidi = YES;
		break;
	    default:
		error("main", "Illegal option: -%s", arg);
	    }
	else
	    mappingsfile = arg;
    }
    
     // Should have processed all arguments by now so print an error if we
     // haven't. 
     
    if (argc != 0 || mappingsfile == NULL) {
	cerr << "\nUsage:\t" << prog << " <Key mappings file>\n"
	     << "\t[-s] [-t <time stamp file>] [-v] [-f] [-i] "
	     << "[-e <exponent>] [-m]\n\n\t[-s] is silent mode--no clipping or"
	     << " audio port underflow\n\terrors are reported.\n"
	     << "\t[-t <time stamp file>] is timestamp mode.  The timings of\n"
	     << "\tyour key strokes (relative to the first key stroke) are\n"
	     << "\twritten to the file <time stamp file>.\n"
	     << "\t[-v] just prints version and compile time information.\n"
	     << "\t[-f] uses a faster mixing algorithm that doesn't check"
	     << " for\n\twrap-around distortion.  (This means if sounds are"
	     << " too\n\tloud the output will be severely distorted.)\n"
	     << "\t[-e <exponent>] sets the exponent that the MIDI fader"
	     << " value\n\twill be raised to.  Default is 1.0, so a"
	     << " straight line fade\n\toccurs.\n"
	     << "\t[-i] turns sound information off so that print out will"
	     << " be\n\tat a minimum.\n"
	     << "\t[-m] interactively prints MIDI values for each "
	     << "channel.\n\n";
	exit(1);
    }

#ifndef DEBUG
    // Set up the terminal to catch keyboard input without echoing. 
    initscr();
    noecho();
    cbreak();
    refresh();
#endif // DEBUG

    // This is where the programme really starts.
    mix mixer(mappingsfile);

    mixer.run(timestampfile);
    fflush(stdout);

#ifndef DEBUG
    // Get out of curses mode. 
    endwin();
#endif // DEBUG

    return 0;
}

//*****************************************************************************

// EOF main.cc 
 
